-- Globals:
-- ========
-- our big global object:
SCoordsAddon = {
	addon_loaded = false,
	seconds_since_update = 0, -- tick counter
	seconds_since_map_update = 0 -- tick counter
}

-- boot frame:
local boot_frame = CreateFrame("Frame")
boot_frame:RegisterEvent("ADDON_LOADED")
boot_frame:SetScript(
  "OnEvent", function(frame,event, addon, ...)
		if frame == boot_frame then
	    SCoordsAddon.init(event, addon);
		end
  end
);

local HEARTBEAT_INTERVAL = 0.5; -- how many seconds per heartbeat of our internal timer


local minimapIcon;
local worldTarget;
local panelOpen = false;
local haveArrived=false;
local dest_x = -1000;
local dest_y = -1000;
local showWorldMapIcon = true;
local zone_dirty = true;

-- Constants:
local MINIMAP_ICON_NAME = "SimpleCoords";
local WORLDMAP_ICON_NAME = "SimpleCoordsW";

local HALF_PI = math.pi / 2;
local ROOT_HALF = math.sqrt(0.5)
-- Bring in HereBeDragons
local hbd = LibStub("HereBeDragons-2.0")
local pins = LibStub("HereBeDragons-Pins-2.0")



-- ----------------------------------
function SimpleCoords_AcceptValue()
--
-- Called when ENTERing new values in the coordinate window
-- Reads the values in SimpleCoords_X and SimpleCoords_Y text input frames (Still defined in the XML)
-- This function is attached to the OnEnterPressed handlers for those frames.

    -- clear any existing icon(s), since we have new coords:
    SimpleCoordsRemoveIcons()

  if (_G["SimpleCoords_X"]:GetText() == "" and _G["SimpleCoords_Y"]:GetText() == "") then
    SimpleCoords_TogglePanel();
  else
    _G["SimpleCoords_X"]:ClearFocus();
    _G["SimpleCoords_Y"]:ClearFocus();
    if (_G["SimpleCoords_X"]:GetText() == "" or _G["SimpleCoords_Y"]:GetText() == "") then
    -- only one field filled in
    else
      PlaySoundFile(567574);
      -- PlaySoundFile("Sound\\interface\\PickUp\\PutDownGems.wav");
      local x_text, y_text;
      x_text = _G["SimpleCoords_X"]:GetText();
      y_text = _G["SimpleCoords_Y"]:GetText();
      y_text = gsub(y_text, ",", "."); -- swap commas for decimals for those European types
      y_text = gsub(y_text, "[%a%c%-%(%)]", ""); -- %a: letters. %c: control characters %-: -
      x_text = gsub(x_text, ",", ".");
      x_text = gsub(x_text, "[%a%c%-%(%)]", ""); -- %a: letters. %c: control characters %-: -
      dest_x=tonumber(x_text)/100.0; -- revert scale back to 0 - 1.0
      dest_y=tonumber(y_text)/100.0;

      -- add the dots to the map(s)
      -- minimap
      local playerZoneX, playerZoneY, uiMapID = SimpleCoords_GetPlayerZoneCoords();

      AddMinimapIcon(minimapIcon, dest_x, dest_y, uiMapID ) --icon, x, y, uiMapID

      -- add world map
      -- DEFAULT_CHAT_FRAME:AddMessage("mapID: " .. mapID ,  0.8, 1.0, 0.5, 1);
      AddWorldMapIcon(worldTarget, uiMapID, dest_x, dest_y);
      haveArrived=false;

    end

  end
end





-- ----------------------------------
function SimpleCoords_TogglePanel()
--
-- Open or close the coordinate window depending on current state
-- With accompanying sound
--
	if(_G["SimpleCoords_X"]:IsShown()) then
	  PlaySoundFile(567515);
		-- PlaySoundFile("Sound\\interface\\uMiniMapClose.wav");

		_G["SimpleCoords_X"]:Hide();
		_G["SimpleCoords_Y"]:Hide();
		_G["SimpleCoords_X"]:ClearFocus();
		_G["SimpleCoords_Y"]:ClearFocus();
    SimpleCoordsRemoveIcons()
		panelOpen=false;

	else
		PlaySoundFile(567529); -- ("Sound\\interface\\uMiniMapOpen.wav");

		_G["SimpleCoords_X"]:Show();
		_G["SimpleCoords_Y"]:Show();
		_G["SimpleCoords_X"]:SetFocus();
		panelOpen=true;
	end

end





function SimpleCoords_MapIcon_Update(self, elapsed)
--
-- Updates the minimap icon
--
-- Uses HEARTBEAT_INTERVAL to limit how often the display is updated
-- params:
-- self
-- elapsed incrementing timer (second resolution)
--

  SCoordsAddon.seconds_since_map_update = SCoordsAddon.seconds_since_map_update + elapsed;

	if (SCoordsAddon.seconds_since_map_update > HEARTBEAT_INTERVAL) then
	  if haveArrived ~= true then
	  -- if the panel is open, but we "have arrived" don't bother to update hidden things

    local on_edge = IsMinimapIconOnEdge(minimapIcon)  --	local edge = Astrolabe:IsIconOnEdge(self)
    local showing_dot = self.dot:IsShown()
    local showing_arrow = self.arrow:IsShown()

    if on_edge then
     -- DEFAULT_CHAT_FRAME:AddMessage("EDGE",  0.8, 1.0, 0.5, 1);
    else
     -- DEFAULT_CHAT_FRAME:AddMessage("UPDATE",  0.8, 1.0, 0.5, 1);
    end

      if (on_edge and not showing_arrow) then
        self.arrow:Show()
        self.dot:Hide()
        -- DEFAULT_CHAT_FRAME:AddMessage("TO ARROW",  0.8, 1.0, 0.5, 1);
      elseif not on_edge and not showing_dot then
        self.dot:Show()
        self.arrow:Hide()
        -- DEFAULT_CHAT_FRAME:AddMessage("TO DOT",  0.8, 1.0, 0.5, 1);
      end


   local angle,distance = pins:GetVectorToIcon(minimapIcon) --	local angle = Astrolabe:GetDirectionToIcon(minimapIcon)
   if (distance ~= nil) then
    if (distance < 10) then
      if (not haveArrived) then
        DEFAULT_CHAT_FRAME:AddMessage("You have arrived.",  0.8, 1.0, 0.5, 1); -- let them now they have arrived
        -- remove the icons
        SimpleCoordsRemoveIcons()
        haveArrived = true
        event_complete = false
      end -- haveArrived
    end -- distance < 10)
   end


      if showing_arrow then
        if (angle ~= nil) then
        angle = angle + math.rad(135) -- math.rad converts degrees to radians 45 past 90 in this case
        -- if they have the rotating minimap enabled, we have to adjust our angle:
          if GetCVar("rotateMinimap") == "1" then
                local minimap_rot = GetPlayerFacing()
                angle = angle - minimap_rot
            end

          local sin,cos = math.sin(angle) * ROOT_HALF, math.cos(angle) * ROOT_HALF
          self.arrow:SetTexCoord(0.5-sin, 0.5+cos, 0.5+cos, 0.5+sin, 0.5-cos, 0.5-sin, 0.5+sin, 0.5-cos)

        else
          self:Hide() -- an error getting the angle to the icon. so just hide it?
        end
      end -- showing_arrow
    end  -- haveArrived
    SCoordsAddon.seconds_since_map_update = 0;
	end
end









-- Tooltips:
--------------

function SimpleCoords_ShowTooltip()
	local tooltipList ={};
	GameTooltip:SetOwner(_G["SimpleCoordsFrame"] , "ANCHOR_CURSOR", -5, 5);
	-- GameTooltip:SetOwner(owner, "anchor"[, +x, +y]);
	--GameTooltip:AddLine("test", .6,1.0,.8); -- GameTooltip:AddLine(name, GameTooltip_UnitColor("player"));\
	GameTooltip:AddLine('SimpleCoords',.6,1.0,.8);
	GameTooltip:AddLine('Click to enter coordinates.');
	GameTooltip:AddLine('Shift-click to move.');
	GameTooltip:AddLine('ENTER to place dot at coordinates.');
	GameTooltip:Show();
end






function SCoordsAddon.HideTooltip()
	GameTooltip:Hide();
end





function SimpleCoordsEnter_ShowTooltip()
	local tooltipList ={};
	GameTooltip:SetOwner(_G["SimpleCoordsFrame"] , "ANCHOR_CURSOR", -5, 5);
	-- GameTooltip:SetOwner(owner, "anchor"[, +x, +y]);
	--GameTooltip:AddLine("test", .6,1.0,.8); -- GameTooltip:AddLine(name, GameTooltip_UnitColor("player"));\
	GameTooltip:AddLine('SimpleCoords',.6,1.0,.8);
	GameTooltip:AddLine('TAB to switch fields.');
	GameTooltip:AddLine('Click current coordinates to close edit.');
	GameTooltip:AddLine('ENTER to place dot at coordinates.');
	GameTooltip:Show();
end




-- command line parameters: (slash command handler)
----------------------------------------------------
function SimpleCoords_CommandLine(msg)
	cmd=string.lower(msg);
end


-- Move this glue to a separate file?
function SimpleCoords_GetPlayerZoneCoords()
    local x, y, UiMapID, UiMapType = hbd:GetPlayerZonePosition();
    return  x, y, UiMapID, UiMapType;
end


function AddMinimapIcon(icon, x, y, uiMapID)
  pins:AddMinimapIconMap(MINIMAP_ICON_NAME, icon, uiMapID, x, y, true, true) -- showInParentZone, floatOnEdge
end

function SCoordsAddon.AddWorldMapIcon(icon, uiMapID, x, y)
  pins:AddWorldMapIconMap(WORLDMAP_ICON_NAME, icon, uiMapID, x, y, HBD_PINS_WORLDMAP_SHOW_PARENT); -- ref, icon, uiMapID, x, y, showFlag)
end


function SCoordsAddon.RemoveIcons()
  -- Remove both the minimap and worldmap icons
  SCoordsAddon.RemoveMinimapIcon(minimapIcon)
  SCoordsAddon.RemoveWorldMapIcon(worldTarget)
end

function SCoordsAddon.RemoveWorldMapIcon(icon)
  pins:RemoveWorldMapIcon(WORLDMAP_ICON_NAME, icon)
end


function SCoordsAddon.RemoveMinimapIcon(icon)
  pins:RemoveMinimapIcon(MINIMAP_ICON_NAME, icon)
end

function SCoordsAddon.IsMinimapIconOnEdge(icon)
  local t = pins:IsMinimapIconOnEdge(icon)
  return t;
end

function SCoordsAddon.GetVectorToIcon(icon)
  -- return anggle and distance
  local angle, distance = pins:GetVectorToIcon(icon);
  return angle, distance;
end




--- Once the addon is loaded, register all our events and create our frames
--
function SCoordsAddon.init(event, addon)
	local need_to_init = false;
	if event == "ADDON_LOADED" and addon == "SimpleCoords" then
	DEFAULT_CHAT_FRAME:AddMessage("HELLLLLO",  0.5, 1.0, 0.5, 1); -- leave for testing

		if ( false == SCoordsAddon.addon_loaded ) then -- only need to load once
			SCoordsAddon.addon_loaded = true;

			-- do we need to init the saved variables?
			if not (SimpleAssistCharVars) then
				need_to_init = true;
			else
				if (nil == SimpleAssistCharVars["version"]) then
					need_to_init = true;
				else
					if SimpleAssistCharVars["version"] < 4 then
						need_to_init = true;
					end
				end
			end
 -- need_to_init = true;

			if (need_to_init) then
				DEFAULT_CHAT_FRAME:AddMessage(SASSTEXT.FIRST_RUN,  0.5, 1.0, 0.5, 1);
				-- never saved, create from scratch
				SimpleAssistCharVars = SCoordsAddon.deepcopy(SCoordsAddon.defaults);
				SCoordsAddon.unsaved_settings = SCoordsAddon.deepcopy(SimpleAssistCharVars);


			end



			-- Register all our events:
			SCoordsAddon.eventframe = CreateFrame("Frame" ); -- the frame to listen to all our events
			local ev = SCoordsAddon.eventframe; -- local shortcut

			local f =  CreateFrame("CheckButton", "SimpleAssistActionButton", UIParent, "ActionBarButtonTemplate");

			f:SetAttribute("type", "macro");
			f:SetAttribute("width", "30");
			f:SetAttribute("height", "30");
			f:SetAttribute("macrotext", "/assist target");
			f:ClearAllPoints();
			f:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 50, 50); -- (point, frame, relativePoint , x, y);
			-- putting the button off-screen to make it invisible
			-- f:SetPoint("BOTTOMRIGHT", "UIParent", "TOPLEFT", 2, -2); -- (point, frame, relativePoint , x, y);


			-- and register our basic events:
			ev:RegisterEvent("PLAYER_ENTERING_WORLD");
			ev:RegisterEvent("CHAT_MSG_CHANNEL");
			ev:RegisterEvent("VARIABLES_LOADED");
			ev:RegisterEvent("UPDATE_BINDINGS");
			ev:RegisterEvent("PLAYER_LEAVE_COMBAT");
			ev:RegisterEvent("PLAYER_REGEN_ENABLED");

			-- self:RegisterEvent("CHAT_MSG_TEXT_EMOTE"); -- for listening
			ev:SetScript("OnEvent", SCoordsAddon.HandleEvent);

			-- Set up our display frame:
	--		SCoordsAddon.alert = CreateFrame("MessageFrame", "SimpleAssistAlertFrame", UIParent );
		--	SCoordsAddon.alert.


			-- Set up the preference panel:
			SimpleAssist_Options_CreatePanel();
		end --SCoordsAddon.addon_loaded
	end -- ADDON_LOADED
end
